==> gneur-0.0.9a.tar.gz: gneur-0.0.9/src/xconfig.c <==
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Originally written by Alphex Kaanoken <kaanoken@crew.org.ru>
 *
 *  NOTE: is a parser functions , initialize and parse
 *
 *  (c) Crew IT Research Labs 2004
 *
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <stdarg.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <gtk/gtkplug.h>
#include <gdk/gdkx.h>

#include "utils.h"
#include "xconfig.h"
#include "error.h"

/*linked list operations*/
app_list *AddItem (app_list * ptr, char *name)
{
	app_list *lp = ptr;
	int id = 0;
	if (ptr != NULL) {
		++id;
		while (ptr->next != NULL) {
			ptr = ptr->next;
			++id;
		}
		ptr->next = (app_list *) _malloc (sizeof (app_list));
		ptr = ptr->next;
		ptr->next = NULL;
		ptr->id = id;
		ptr->name = strdup ((const char *) name);
		return lp;
	}
	else {
		ptr = (app_list *) _malloc (sizeof (app_list));
		ptr->next = NULL;
		ptr->id = id;
		ptr->name = strdup ((const char *) name);
		return ptr;
	}
}

app_list *DelItem (app_list * ptr, int id)
{
	app_list *prev = NULL;
	app_list *curr;
	curr = ptr;

	while (curr != NULL && curr->id != id) {
		prev = curr;
		curr = curr->next;
	}
	if (curr == NULL)
		return ptr;
	if (curr == ptr)
		ptr = curr->next;
	else
		prev->next = curr->next;
	free (curr->name);
	free (curr);
	return ptr;
}

app_list *FreeItems (app_list * ptr)
{
	if (ptr != NULL) {
		while (ptr != NULL)
			ptr = DelItem (ptr, ptr->id);
	}
	return ptr;
}

/*copy file,read file and parse it functions*/
int XConfigCopyFile (const char *src_path, const char *dst_path, int *new_dst)
{
	char *buf;
	int buf_size;
	int dest_desc;
	int source_desc;
	struct stat sb;
	struct stat src_open_sb;
	char *cp;
	int *ip;
	int return_val = 0;
	off_t n_read_total = 0;
	int last_write_made_hole = 0;
	int make_holes = 0;

	source_desc = open (src_path, O_RDONLY);	/*open file */
	if (source_desc < 0) {
		fprintf (stderr, "ERR: Can't open source file: %s\n", src_path);
		return -1;
	}
	/* We cool hackers - let's test for copy or moving
	 * our file and open it.  */
	if (!new_dst) {
		printf ("ERR:\n");
		dest_desc = open (dst_path, O_WRONLY | O_CREAT, S_IRWXU | S_IRGRP | S_IROTH);
	}
	else {
		dest_desc = open (dst_path, O_WRONLY | O_TRUNC, S_IRWXU | S_IRGRP | S_IROTH);
		/*TODO: unlink(remove) file if it's exists */
	}

	if (dest_desc < 0) {
		fprintf (stderr, "Can't create new file: %s\n", dst_path);
		return_val = -1;
		goto close_src_desc;	/*goto rulezz! =) */
	}

	/*Determine the optimal buffer size */
	if (fstat (dest_desc, &sb)) {
		fprintf (stderr, "Can't read status of: %s\n", dst_path);
		return_val = -1;
		goto close_src_and_dst_desc;
	}
	buf_size = 2048;	/*so, for compatibility with win32 I think better is 2048 */

	/*Make buffer with space at end */
	buf = malloc (buf_size + sizeof (int));
	/*so and try to copy */
	for (;;) {
		ssize_t n_read = read (source_desc, buf, buf_size);
		if (n_read < 0) {
			fprintf (stderr, "Error while reading source: %s", src_path);
			return_val = -1;
			goto close_src_and_dst_desc;
		}
		if (n_read == 0)
			break;	/*the end (EOF) */
		n_read_total += n_read;	/*inc counter */
		ip = 0;		/*reset */
		if (make_holes) {	/*TODO: make holes and zeros determine! */
			buf[n_read] = 1;	/*stop loop|not */
			/*Find first specific;-) symbol */
			ip = (int *) buf;
			while (*ip++ == 0)	/*reduce */
				;
			/*Find first non-zero symbol */
			cp = (char *) (ip - 1);
			while (*cp++ == 0)	/*reduce */
				;
			/* if we found its it rules,we can make a hole */
			if (cp > buf + n_read) {
				/*make hole-sucks with lseek */
				if (lseek (dest_desc, (off_t) n_read, SEEK_CUR) < 0L) {
					fprintf (stderr, "Can't seek in \n");
					return_val = -1;
					goto close_src_and_dst_desc;
				}
				last_write_made_hole = 1;
			}
			else
				/*reset we don't need ih hole */
				ip = 0;
		}
		if (ip == 0) {
			size_t n = n_read;
			if (write (dest_desc, buf, n) != n) {
				fprintf (stderr, "Can't write to file: %s\n", dst_path);
				return_val = -1;
				goto close_src_and_dst_desc;
			}
			last_write_made_hole = 0;
		}
	}
	/*TODO: if we make a hole in last write, we need put to file
	 * something at end*/
      close_src_and_dst_desc:
	if (close (dest_desc) < 0) {
		fprintf (stderr, "Can't close destination!\n");
		return_val = -1;
	}
      close_src_desc:
	if (close (source_desc) < 0) {
		fprintf (stderr, "Can't close the source!\n");
		return_val = -1;
	}
	free (buf);
	/*just end - return the value */
	return return_val;
}

xconf *ParseLine (xconf * ptr, const char *line)
{
	char *f_word = (char *) _malloc (strlen (line) + 1);
	int cp = 0;
	int j = -1;
	int i = 0;
	status action, key, modifier;
	
	while (line[i] != ' ') {
		f_word[i] = line[i];
		++i;
	}

	f_word[i] = NULLSYM;

	if (!strcmp (f_word, "DefaultMode"))
		cp = 1;
	if (!strcmp (f_word, "ConvPoint"))
		cp = 2;
	if (!strcmp (f_word, "AddApp"))
		cp = 3;
	if (!strcmp (f_word, "AddBind"))
		cp = 4;
	if (!strcmp (f_word, "ScanDelay"))
		cp = 5;
	if (!strcmp (f_word, "LangDetectMode"))
		cp = 6;
	if (!cp)
		fprintf (stderr, "[WRN]: Unrecognized option '%s' in config file!\n", f_word);

	f_word = memset (f_word, NULLSYM, strlen (line) + 1);

	while (line[++i] != NULLSYM)
		f_word[++j] = line[i];

	switch (cp) {
		case 1:
			if ((ptr->DefaultMode = atoi (f_word)) > 1
			    || (ptr->DefaultMode = atoi (f_word)) < 0) {
				fprintf (stderr,
					 "[WRN]: Bad value '%d' (use 0 or 1 for default mode), using default.\n",
					 atoi (f_word));
				ptr->DefaultMode = 0;
			}
			break;
		case 2:
			if ((ptr->ConvPoint = atoi (f_word)) < 30
			    || (ptr->ConvPoint = atoi (f_word)) > 300) {
				fprintf (stderr,
					 "[WRN]: Bad value '%d' (values must be in 30-300), using default.\n",
					 atoi (f_word));
				ptr->ConvPoint = 55;
			}
			break;
		case 3:
			ptr->alist = AddItem (ptr->alist, f_word);
			break;
		case 4:
			action = atoi(f_word);
			i = 0;
			while( (f_word[i] != NULLSYM) && (f_word[i++] != ' ') ) ;
			key = atoi(&f_word[i]);
			while( (f_word[i] != NULLSYM) && (f_word[i++] != ' ') ) ;
			modifier = atoi(&f_word[i]);
			if( (f_word[i] == NULLSYM) || (action >= DEFAULT_BIND_TABLE_SIZE)
				|| (action < 0)	|| (key < 0) || (modifier < 0) ){
				send_error(WARNING, "Bad binding values. Set to default\n", NULL);
			}
			else {
				ptr->btable[action].key = key;
				ptr->btable[action].key_modifier = modifier;
			}
			break;
		case 5:
			if( ((ptr->ScaningDelay = atoi (f_word)) < 1000 )
					|| (ptr->ScaningDelay > 30000 ) ){
				send_error(WARNING,
								"ScaningDelay is set to not recommended value %d \n"\
								"(Recommended: 1000-30000). Set to default.\n",
								&(ptr->ScaningDelay) );
				ptr->ScaningDelay = DEFAULT_SCANNING_DELAY;
			}
			break;
		case 6:
			if ((ptr->LangDetectMode = atoi (f_word)) > 1
			    || (ptr->LangDetectMode = atoi (f_word)) < 0) {
				fprintf (stderr,
					 "[WRN]: Bad value '%d' (use 0 or 1 for lang detection mode), using default.\n",
					 atoi (f_word));
				ptr->LangDetectMode = 0;
			}
			break;	
		default:
			fprintf (stderr, "[WRN]: Ignored value '%s'.\n", f_word);
			break;
	}

	free (f_word);
	return ptr;
}

xconf *XConfigInit (xconf * ptr)
{
	ptr = (struct _xconf *) _malloc (sizeof (xconf));
	ptr->DefaultMode = AUTO_BY_KEYLOG;
	ptr->CurrentMode = AUTO_BY_KEYLOG;
	ptr->ConvPoint = 55;	/*default values */
	ptr->LangDetectMode = 0;  /*old algorithm*/
	ptr->ScaningDelay = DEFAULT_SCANNING_DELAY;
	ptr->alist = NULL;
	ptr->rudict = NULL;
	ptr->endict = NULL;
	memcpy(ptr->btable, DEFAULT_BT, DEFAULT_BIND_TABLE_SIZE*sizeof(bind));
	
	return ptr;
}

xconf *XConfigLoad (xconf * ptr, const char *fl_name)
{
	FILE *stream = NULL;
	int sym = 0;
	int i = -1;
	/*States of Parser */
	int isRead = 1;
	int t = 0;
	
	char *t_word = (char *) _malloc (258);

	if ((stream = fopen (fl_name, "r")) != NULL) {
		while ((sym = fgetc (stream)) != EOF) {
			switch (sym) {
				case '#':
					isRead = 0;
					break;
				case '\n':
					if ((t = strlen ((const char *) t_word)) != 0)
						isRead = 10;
					else
						isRead = 0;
					if (isRead) {
						t_word[++i] = NULLSYM;
						ptr = ParseLine (ptr, (const char *)
								 t_word);
						t_word = memset (t_word, NULLSYM, 258);
						i = -1;
					}
					isRead = 1;
					break;
				default:
					if (isRead)
						t_word[++i] = (unsigned char) sym;
					break;
			}

			if (i > 255) {
				fprintf (stderr,
					 "[ERR]: Configuration file '%s' is bad!\nExit(1)\n");
				free (t_word);
				fclose (stream);
				exit (1);
			}
		}
		fclose (stream);
		free (t_word);

		return ptr;
	}
	else {
		fprintf (stderr,
			 "[WRN]: Cannot read config file: %s, using default settings!\n", fl_name);
		free (t_word);
		return ptr;
	}
}

xconf *XConfigFree (xconf * ptr)
{
	ptr->alist = FreeItems (ptr->alist);
	free (ptr);
	return ptr;		/*just free */
}


char *XDictionaryLoad (char *ptr, const char *dict_name)
{
	FILE *stream = NULL;
	int sym = 0;
	int i = 0;
	// States of Parser 
	int isRead = 1;
	int t = 0;
	char *t_word = (char *) _malloc (258);
	char *word = (char *) _malloc (1);
	// init russian dictionary
	if ((stream = fopen (dict_name, "r")) != NULL) {
		word[0]='\0';
		t_word[i] = '|';
		t_word[++i] = '\n';
		while ((sym = fgetc (stream)) != EOF) {
			switch (sym) {
				case '#':
					isRead = 0;
					break;
				case '\n':
					if ((t = strlen ((const char *) t_word)) != 0)
						isRead = 10;
					else
						isRead = 0;
					if (isRead) {
						t_word[i] = '|';
						t_word[++i] = NULLSYM;
						word = realloc(word, strlen((const char *)t_word) + strlen((const char *)word) + 1);
						strcat(word, t_word);
						//printf ("'%s'\n", word);
						t_word = memset (t_word, NULLSYM, 258);
						i = 0;
					}
					isRead = 1;
					break;
				default:
					if (isRead) {
						t_word[i] = (unsigned char) sym;
						t_word[++i] = NULLSYM;
						//printf ("'%s'\n", t_word);
					}
					break;
			}

			if (i > 255) {
				fprintf (stderr,
					 "[ERR]: Russian Dictionary file '%s' is bad!\nExit(1)\n");
				free (t_word);
				fclose (stream);
				exit (1);
			}
		}
		fclose (stream);	
		free (t_word);
		return word;
	}
	else {
		fprintf (stderr,
			 "[WRN]: Cannot read russian dictionary file: %s, russian dictionary not use!\n", dict_name);
		if (t_word != NULL) 
			free (t_word);
		return ptr;
	}
}

==> gneur-0.1.0.tar.gz: gneur-0.1.0/src/xconfig.c <==
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Originally written by Alphex Kaanoken <kaanoken@crew.org.ru>
 *
 *  NOTE: is a parser functions , initialize and parse
 *
 *  (c) Crew IT Research Labs 2004
 *
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <stdarg.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <gtk/gtkplug.h>
#include <gdk/gdkx.h>

#include "utils.h"
#include "xconfig.h"
#include "error.h"

/*linked list operations*/
app_list *AddItem (app_list * ptr, char *name)
{
	app_list *lp = ptr;
	int id = 0;
	if (ptr != NULL) {
		++id;
		while (ptr->next != NULL) {
			ptr = ptr->next;
			++id;
		}
		ptr->next = (app_list *) _malloc (sizeof (app_list));
		ptr = ptr->next;
		ptr->next = NULL;
		ptr->id = id;
		ptr->name = strdup ((const char *) name);
		return lp;
	}
	else {
		ptr = (app_list *) _malloc (sizeof (app_list));
		ptr->next = NULL;
		ptr->id = id;
		ptr->name = strdup ((const char *) name);
		return ptr;
	}
}

app_list *DelItem (app_list * ptr, int id)
{
	app_list *prev = NULL;
	app_list *curr;
	curr = ptr;

	while (curr != NULL && curr->id != id) {
		prev = curr;
		curr = curr->next;
	}
	if (curr == NULL)
		return ptr;
	if (curr == ptr)
		ptr = curr->next;
	else
		prev->next = curr->next;
	free (curr->name);
	free (curr);
	return ptr;
}

app_list *FreeItems (app_list * ptr)
{
	if (ptr != NULL) {
		while (ptr != NULL)
			ptr = DelItem (ptr, ptr->id);
	}
	return ptr;
}

/*copy file,read file and parse it functions*/
int XConfigCopyFile (const char *src_path, const char *dst_path, int *new_dst)
{
	char *buf;
	int buf_size;
	int dest_desc;
	int source_desc;
	struct stat sb;
	struct stat src_open_sb;
	char *cp;
	int *ip;
	int return_val = 0;
	off_t n_read_total = 0;
	int last_write_made_hole = 0;
	int make_holes = 0;

	source_desc = open (src_path, O_RDONLY);	/*open file */
	if (source_desc < 0) {
		fprintf (stderr, "ERR: Can't open source file: %s\n", src_path);
		return -1;
	}
	/* We cool hackers - let's test for copy or moving
	 * our file and open it.  */
	if (!new_dst) {
		printf ("ERR:\n");
		dest_desc = open (dst_path, O_WRONLY | O_CREAT, S_IRWXU | S_IRGRP | S_IROTH);
	}
	else {
		dest_desc = open (dst_path, O_WRONLY | O_TRUNC, S_IRWXU | S_IRGRP | S_IROTH);
		/*TODO: unlink(remove) file if it's exists */
	}

	if (dest_desc < 0) {
		fprintf (stderr, "Can't create new file: %s\n", dst_path);
		return_val = -1;
		goto close_src_desc;	/*goto rulezz! =) */
	}

	/*Determine the optimal buffer size */
	if (fstat (dest_desc, &sb)) {
		fprintf (stderr, "Can't read status of: %s\n", dst_path);
		return_val = -1;
		goto close_src_and_dst_desc;
	}
	buf_size = 2048;	/*so, for compatibility with win32 I think better is 2048 */

	/*Make buffer with space at end */
	buf = malloc (buf_size + sizeof (int));
	/*so and try to copy */
	for (;;) {
		ssize_t n_read = read (source_desc, buf, buf_size);
		if (n_read < 0) {
			fprintf (stderr, "Error while reading source: %s", src_path);
			return_val = -1;
			goto close_src_and_dst_desc;
		}
		if (n_read == 0)
			break;	/*the end (EOF) */
		n_read_total += n_read;	/*inc counter */
		ip = 0;		/*reset */
		if (make_holes) {	/*TODO: make holes and zeros determine! */
			buf[n_read] = 1;	/*stop loop|not */
			/*Find first specific;-) symbol */
			ip = (int *) buf;
			while (*ip++ == 0)	/*reduce */
				;
			/*Find first non-zero symbol */
			cp = (char *) (ip - 1);
			while (*cp++ == 0)	/*reduce */
				;
			/* if we found its it rules,we can make a hole */
			if (cp > buf + n_read) {
				/*make hole-sucks with lseek */
				if (lseek (dest_desc, (off_t) n_read, SEEK_CUR) < 0L) {
					fprintf (stderr, "Can't seek in \n");
					return_val = -1;
					goto close_src_and_dst_desc;
				}
				last_write_made_hole = 1;
			}
			else
				/*reset we don't need ih hole */
				ip = 0;
		}
		if (ip == 0) {
			size_t n = n_read;
			if (write (dest_desc, buf, n) != n) {
				fprintf (stderr, "Can't write to file: %s\n", dst_path);
				return_val = -1;
				goto close_src_and_dst_desc;
			}
			last_write_made_hole = 0;
		}
	}
	/*TODO: if we make a hole in last write, we need put to file
	 * something at end*/
      close_src_and_dst_desc:
	if (close (dest_desc) < 0) {
		fprintf (stderr, "Can't close destination!\n");
		return_val = -1;
	}
      close_src_desc:
	if (close (source_desc) < 0) {
		fprintf (stderr, "Can't close the source!\n");
		return_val = -1;
	}
	free (buf);
	/*just end - return the value */
	return return_val;
}

xconf *ParseLine (xconf * ptr, const char *line)
{
	char *f_word = (char *) _malloc (strlen (line) + 1);
	int cp = 0;
	int j = -1;
	int i = 0;
	status action, key, modifier;
	
	while (line[i] != ' ') {
		f_word[i] = line[i];
		++i;
	}

	f_word[i] = NULLSYM;

	if (!strcmp (f_word, "DefaultMode"))
		cp = 1;
	if (!strcmp (f_word, "ConvPoint"))
		cp = 2;
	if (!strcmp (f_word, "AddApp"))
		cp = 3;
	if (!strcmp (f_word, "AddBind"))
		cp = 4;
	if (!strcmp (f_word, "ScanDelay"))
		cp = 5;
	if (!strcmp (f_word, "LangDetectMode"))
		cp = 6;
	if (!cp)
		fprintf (stderr, "[WRN]: Unrecognized option '%s' in config file!\n", f_word);

	f_word = memset (f_word, NULLSYM, strlen (line) + 1);

	while (line[++i] != NULLSYM)
		f_word[++j] = line[i];

	switch (cp) {
		case 1:
			if ((ptr->DefaultMode = atoi (f_word)) > 1
			    || (ptr->DefaultMode = atoi (f_word)) < 0) {
				fprintf (stderr,
					 "[WRN]: Bad value '%d' (use 0 or 1 for default mode), using default.\n",
					 atoi (f_word));
				ptr->DefaultMode = 0;
			}
			break;
		case 2:
			if ((ptr->ConvPoint = atoi (f_word)) < 30
			    || (ptr->ConvPoint = atoi (f_word)) > 300) {
				fprintf (stderr,
					 "[WRN]: Bad value '%d' (values must be in 30-300), using default.\n",
					 atoi (f_word));
				ptr->ConvPoint = 55;
			}
			break;
		case 3:
			ptr->alist = AddItem (ptr->alist, f_word);
			break;
		case 4:
			action = atoi(f_word);
			i = 0;
			while( (f_word[i] != NULLSYM) && (f_word[i++] != ' ') ) ;
			key = atoi(&f_word[i]);
			while( (f_word[i] != NULLSYM) && (f_word[i++] != ' ') ) ;
			modifier = atoi(&f_word[i]);
			if( (f_word[i] == NULLSYM) || (action >= DEFAULT_BIND_TABLE_SIZE)
				|| (action < 0)	|| (key < 0) || (modifier < 0) ){
				send_error(WARNING, "Bad binding values. Set to default\n", NULL);
			}
			else {
				ptr->btable[action].key = key;
				ptr->btable[action].key_modifier = modifier;
			}
			break;
		case 5:
			if( ((ptr->ScaningDelay = atoi (f_word)) < 1000 )
					|| (ptr->ScaningDelay > 30000 ) ){
				send_error(WARNING,
								"ScaningDelay is set to not recommended value %d \n"\
								"(Recommended: 1000-30000). Set to default.\n",
								&(ptr->ScaningDelay) );
				ptr->ScaningDelay = DEFAULT_SCANNING_DELAY;
			}
			break;
		case 6:
			if ((ptr->LangDetectMode = atoi (f_word)) > 1
			    || (ptr->LangDetectMode = atoi (f_word)) < 0) {
				fprintf (stderr,
					 "[WRN]: Bad value '%d' (use 0 or 1 for lang detection mode), using default.\n",
					 atoi (f_word));
				ptr->LangDetectMode = 0;
			}
			break;	
		default:
			fprintf (stderr, "[WRN]: Ignored value '%s'.\n", f_word);
			break;
	}

	free (f_word);
	return ptr;
}

xconf *XConfigInit (xconf * ptr)
{
	//bind DEFAULT_BT[] = { {NONE_KEY,NONE_KEY},{PAUSE_KEY, NONE_KEY},{PAUSE_KEY, CONTROL_KEY},{SYS_RQ_KEY, CONTROL_KEY}, {PAUSE_KEY, SHIFT_KEY}};
	bind DEFAULT_BT[] = { {0,0}, {1,0}, {1,11}, {3,11}, {1,10}};

	ptr = (struct _xconf *) _malloc (sizeof (xconf));
	ptr->DefaultMode = AUTO_BY_KEYLOG;
	ptr->CurrentMode = AUTO_BY_KEYLOG;
	ptr->ConvPoint = 55;	/*default values */
	ptr->LangDetectMode = 1;  /*new algorithm*/
	ptr->ScaningDelay = DEFAULT_SCANNING_DELAY;
	ptr->alist = NULL;
	ptr->rudict = NULL;
	ptr->endict = NULL;
	memcpy(ptr->btable, DEFAULT_BT, DEFAULT_BIND_TABLE_SIZE*sizeof(bind));
	
	return ptr;
}

xconf *XConfigLoad (xconf * ptr, const char *fl_name)
{
	FILE *stream = NULL;
	int sym = 0;
	int i = -1;
	/*States of Parser */
	int isRead = 1;
	int t = 0;
	
	char *t_word = (char *) _malloc (258);

	if ((stream = fopen (fl_name, "r")) != NULL) {
		while ((sym = fgetc (stream)) != EOF) {
			switch (sym) {
				case '#':
					isRead = 0;
					break;
				case '\n':
					if ((t = strlen ((const char *) t_word)) != 0)
						isRead = 10;
					else
						isRead = 0;
					if (isRead) {
						t_word[++i] = NULLSYM;
						ptr = ParseLine (ptr, (const char *)
								 t_word);
						t_word = memset (t_word, NULLSYM, 258);
						i = -1;
					}
					isRead = 1;
					break;
				default:
					if (isRead)
						t_word[++i] = (unsigned char) sym;
					break;
			}

			if (i > 255) {
				fprintf (stderr,
					 "[ERR]: Configuration file '%s' is bad!\nExit(1)\n");
				free (t_word);
				fclose (stream);
				exit (1);
			}
		}
		fclose (stream);
		free (t_word);

		return ptr;
	}
	else {
		fprintf (stderr,
			 "[WRN]: Cannot read config file: %s, using default settings!\n", fl_name);
		free (t_word);
		return ptr;
	}
}

xconf *XConfigFree (xconf * ptr)
{
	ptr->alist = FreeItems (ptr->alist);
	free (ptr);
	return ptr;		/*just free */
}


char *XDictionaryLoad (char *ptr, const char *dict_name)
{
	FILE *stream = NULL;
	int sym = 0;
	int i = 0;
	// States of Parser 
	int isRead = 1;
	int t = 0;
	char *t_word = (char *) _malloc (258);
	char *word = (char *) _malloc (1);
	// init russian dictionary
	if ((stream = fopen (dict_name, "r")) != NULL) {
		word[0]='\0';
		t_word[i] = '|';
		t_word[++i] = '\n';
		while ((sym = fgetc (stream)) != EOF) {
			switch (sym) {
				case '#':
					isRead = 0;
					break;
				case '\n':
					if ((t = strlen ((const char *) t_word)) != 0)
						isRead = 10;
					else
						isRead = 0;
					if (isRead) {
						t_word[i] = '|';
						t_word[++i] = NULLSYM;
						word = realloc(word, strlen((const char *)t_word) + strlen((const char *)word) + 1);
						strcat(word, t_word);
						//printf ("'%s'\n", word);
						t_word = memset (t_word, NULLSYM, 258);
						i = 0;
					}
					isRead = 1;
					break;
				default:
					if (isRead) {
						t_word[i] = (unsigned char) sym;
						t_word[++i] = NULLSYM;
						//printf ("'%s'\n", t_word);
					}
					break;
			}

			if (i > 255) {
				fprintf (stderr,
					 "[ERR]: Russian Dictionary file '%s' is bad!\nExit(1)\n");
				free (t_word);
				fclose (stream);
				exit (1);
			}
		}
		fclose (stream);	
		free (t_word);
		return word;
	}
	else {
		fprintf (stderr,
			 "[WRN]: Cannot read russian dictionary file: %s, russian dictionary not use!\n", dict_name);
		if (t_word != NULL) 
			free (t_word);
		return ptr;
	}
}
